@@ -1,9 +1,9 @@
-# @(#)$Id: Programs.pm 256 2013-03-02 14:59:40Z pjf $
+# @(#)$Id: Programs.pm 261 2013-03-04 13:44:59Z pjf $
 
 package Class::Usul::Programs;
 
 use attributes ();
-use version; our $VERSION = qv( sprintf '0.12.%d', q$Rev: 256 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.12.%d', q$Rev: 261 $ =~ /\d+/gmx );
 
 use Class::Inspector;
 use Class::Usul::IPC;
@@ -302,7 +302,7 @@ sub output {
    return;
 }
 
-sub print_usage_text { # Required to stop MX::Getopt from printing usage (2)
+sub print_usage_text { # Required to stop MX::Getopt from printing usage
 }
 
 sub quiet {
@@ -330,19 +330,19 @@ sub run {
       try {
          defined ($rv = $self->$method( @{ $params } ))
             or throw error => 'Method [_1] return value undefined',
-                     args  => [ $method ];
+                     args  => [ $method ], rv => UNDEFINED_RV;
       }
-      catch { $rv = $self->_catch_run_exception( $_ ) };
+      catch { $rv = $self->_catch_run_exception( $method, $_ ) };
    }
    else {
       $self->error( 'Class '.(blessed $self)." method ${method} not found" );
       $rv = UNDEFINED_RV;
    }
 
-   if    (defined $rv and $rv >  OK) { $self->output( "Terminated code ${rv}" )}
-   elsif (defined $rv and $rv == OK) {
+   if (defined $rv and $rv == OK) {
       $self->output( 'Finished in '.elapsed.' seconds' );
    }
+   elsif (defined $rv and $rv > OK) { $self->output( "Terminated code ${rv}" ) }
    else {
       not defined $rv and $rv = UNDEFINED_RV
          and $self->error( "Method ${method} error uncaught/rv undefined" );
@@ -430,7 +430,13 @@ sub _build__os {
 }
 
 sub _catch_run_exception {
-   my ($self, $error) = @_; my $e = exception $error or return UNDEFINED_RV;
+   my ($self, $method, $error) = @_; my $e = exception $error;
+
+   unless ($e) {
+      $self->error( 'Method [_1] exception without error',
+                    { args => [ $method ] } );
+      return UNDEFINED_RV;
+   }
 
    $e->out and $self->output( $e->out );
    $self->error( $e->error, { args => $e->args } );
@@ -467,9 +473,6 @@ sub _get_run_method {
    return $method;
 }
 
-sub _getopt_full_usage { # Required to stop MX::Getopt from printing usage (1)
-}
-
 sub _man_page_from {
    my ($self, $src) = @_; my $cfg = $self->config;
 
@@ -746,7 +749,7 @@ Class::Usul::Programs - Provide support for command line programs
 
 =head1 Version
 
-This document describes Class::Usul::Programs version 0.12.$Revision: 256 $
+This document describes Class::Usul::Programs version 0.12.$Revision: 261 $
 
 =head1 Synopsis
 
